from django_peeringdb.client_adaptor import setup


def pytest_configure():
    setup.configure(
        DATABASE_ENGINE="django.db.backends.sqlite3",
        DATABASES={
            "default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:",}
        },
        PEERINGDB_SYNC_STRIP_TZ=True,
        SECRET_KEY="s3cr3t",
    )



from django.utils.translation import gettext_lazy as _

MEDIA = (("Ethernet", _("Ethernet")), ("ATM", _("ATM")), ("Multiple", _("Multiple")))

POC_ROLES = (
    ("Abuse", _("Abuse")),
    ("Maintenance", _("Maintenance")),
    ("Policy", _("Policy")),
    ("Technical", _("Technical")),
    ("NOC", _("NOC")),
    ("Public Relations", _("Public Relations")),
    ("Sales", _("Sales")),
)


POLICY_GENERAL = (
    ("Open", _("Open")),
    ("Selective", _("Selective")),
    ("Restrictive", _("Restrictive")),
    ("No", _("No")),
)
POLICY_LOCATIONS = (
    ("Not Required", _("Not Required")),
    ("Preferred", _("Preferred")),
    ("Required - US", _("Required - US")),
    ("Required - EU", _("Required - EU")),
    ("Required - International", _("Required - International")),
)
POLICY_CONTRACTS = (
    ("Not Required", _("Not Required")),
    ("Private Only", _("Private Only")),
    ("Required", _("Required")),
)

PROTOCOLS = (
    ("IPv4", _("IPv4")),
    ("IPv6", _("IPv6")),
)

RATIOS = (
    ("", _("Not Disclosed")),
    ("Not Disclosed", _("Not Disclosed")),
    ("Heavy Outbound", _("Heavy Outbound")),
    ("Mostly Outbound", _("Mostly Outbound")),
    ("Balanced", _("Balanced")),
    ("Mostly Inbound", _("Mostly Inbound")),
    ("Heavy Inbound", _("Heavy Inbound")),
)

REGIONS = (
    ("North America", _("North America")),
    ("Asia Pacific", _("Asia Pacific")),
    ("Europe", _("Europe")),
    ("South America", _("South America")),
    ("Africa", _("Africa")),
    ("Australia", _("Australia")),
    ("Middle East", _("Middle East")),
)

SCOPES = (
    ("", _("Not Disclosed")),
    ("Not Disclosed", _("Not Disclosed")),
    ("Regional", _("Regional")),
    ("North America", _("North America")),
    ("Asia Pacific", _("Asia Pacific")),
    ("Europe", _("Europe")),
    ("South America", _("South America")),
    ("Africa", _("Africa")),
    ("Australia", _("Australia")),
    ("Middle East", _("Middle East")),
    ("Global", _("Global")),
)

TRAFFIC = (
    ("", _("Not Disclosed")),
    ("0-20Mbps", _("0-20Mbps")),
    ("20-100Mbps", _("20-100Mbps")),
    ("100-1000Mbps", _("100-1000Mbps")),
    ("1-5Gbps", _("1-5Gbps")),
    ("5-10Gbps", _("5-10Gbps")),
    ("10-20Gbps", _("10-20Gbps")),
    ("20-50Gbps", _("20-50Gbps")),
    ("50-100Gbps", _("50-100Gbps")),
    ("100-200Gbps", _("100-200Gbps")),
    ("200-300Gbps", _("200-300Gbps")),
    ("300-500Gbps", _("300-500Gbps")),
    ("500-1000Gbps", _("500-1000Gbps")),
    ("1-5Tbps", _("1-5Tbps")),
    ("5-10Tbps", _("5-10Tbps")),
    ("10-20Tbps", _("10-20Tbps")),
    ("20-50Tbps", _("20-50Tbps")),
    ("50-100Tbps", _("50-100Tbps")),
    ("100+Tbps", _("100+Tbps")),
)

NET_TYPES = (
    ("", _("Not Disclosed")),
    ("Not Disclosed", _("Not Disclosed")),
    ("NSP", _("NSP")),
    ("Content", _("Content")),
    ("Cable/DSL/ISP", _("Cable/DSL/ISP")),
    ("Enterprise", _("Enterprise")),
    ("Educational/Research", _("Educational/Research")),
    ("Non-Profit", _("Non-Profit")),
    ("Route Server", _("Route Server")),
)

VISIBILITY = (
    ("Private", _("Private")),
    #                   ('Peers', _('Peers')),
    ("Users", _("Users")),
    ("Public", _("Public")),
)



from django.test import TestCase
from django.utils.dateparse import parse_datetime
from django.utils.timezone import make_aware, make_naive, utc


class TimeZoneTests(TestCase):
    """ test time zone handling """

    def test_conversion(self):
        input = "2020-02-04T20:34:14Z"
        parsed = parse_datetime(input)

        assert parsed.year == 2020
        assert parsed.month == 2
        assert parsed.day == 4
        assert parsed.hour == 20
        assert parsed.minute == 34
        assert parsed.utcoffset().seconds == 0

        stripped = parsed.replace(tzinfo=None)

        assert stripped.year == 2020
        assert stripped.month == 2
        assert stripped.day == 4
        assert stripped.hour == 20
        assert stripped.minute == 34
        assert stripped.utcoffset() is None

        aware = make_aware(stripped)

        assert aware.year == 2020
        assert aware.month == 2
        assert aware.day == 4
        assert aware.hour == 20
        assert aware.minute == 34
        assert aware.utcoffset().seconds == 0

        naive = make_naive(aware, timezone=utc)

        assert naive.year == 2020
        assert naive.month == 2
        assert naive.day == 4
        assert naive.hour == 20
        assert naive.minute == 34
        assert naive.utcoffset() is None



# mock backend classes from peeringdb-py client

from django_peeringdb.models import all_models


def reftag_to_cls(fn):
    return fn


class Resource(object):
    def __init__(self, tag):
        self.tag = tag


class Interface(object):
    REFTAG_RESOURCE = dict(
        [(model.HandleRef.tag, Resource(model.HandleRef.tag)) for model in all_models]
    )

    def get_resource(self, concrete):
        return self.REFTAG_RESOURCE[concrete.HandleRef.tag]



# mock resource classes from peeringdb-py client


class Organization(object):
    pass


class Facility(object):
    pass


class Network(object):
    pass


class InternetExchange(object):
    pass


class InternetExchangeFacility(object):
    pass


class InternetExchangeLan(object):
    pass


class InternetExchangeLanPrefix(object):
    pass


class NetworkFacility(object):
    pass


class NetworkIXLan(object):
    pass


class NetworkContact(object):
    pass



import pytest
import datetime

from django.test import TestCase
import django_peeringdb.models


class CoreTests(TestCase):
    """ test core functionality """

    def test_tag_dict(self):
        tag_dict = django_peeringdb.models.tag_dict
        assert len(tag_dict)
        for k, v in list(tag_dict.items()):
            assert k == v._handleref.tag



import sys
import datetime
from collections import defaultdict
from decimal import Decimal

import pytest
import calendar

from django.db import IntegrityError
from django.db.transaction import atomic as atomic_transaction
import django.core.exceptions

import tests.peeringdb_mock

sys.modules["peeringdb"] = sys.modules["tests.peeringdb_mock"]


import django_peeringdb.models as models

from django_peeringdb.client_adaptor.load import database_settings

from django_peeringdb.client_adaptor.backend import Backend


def test_database_settings():
    db_config = database_settings(
        {
            "engine": "sqlite",
            "name": "test.sqlite3",
            "host": "localhost",
            "port": 1234,
            "user": "test_user",
            "password": "test_password",
            "drop": "this",
        }
    )

    expected = {
        "ENGINE": "django.db.backends.sqlite",
        "NAME": "test.sqlite3",
        "HOST": "localhost",
        "PORT": 1234,
        "USER": "test_user",
        "PASSWORD": "test_password",
    }

    assert db_config == expected


def test_backend_setup():
    Backend.setup()
    for model in models.all_models:
        for field in model._meta.fields:
            if field.name in ["created", "updated"]:
                assert field.auto_now_add == False
                assert field.auto_now == False


def test_atomic_transaction():
    assert isinstance(Backend.atomic_transaction(), type(atomic_transaction()))


def test_validation_error():
    assert Backend.validation_error() == django.core.exceptions.ValidationError


def test_object_missing_error():
    assert Backend.object_missing_error() == django.core.exceptions.ObjectDoesNotExist
    assert (
        Backend.object_missing_error(models.Organization)
        == models.Organization.DoesNotExist
    )


@pytest.mark.django_db
def test_last_change(db):
    backend = Backend()
    now = datetime.datetime.now()
    now_t = int(calendar.timegm(now.timetuple()))
    models.Organization.objects.create(name="Test org", created=now, updated=now)
    assert backend.last_change(models.Organization) == now_t
    assert backend.last_change(models.Network) == 0


@pytest.mark.django_db
def test_get_object(db):
    backend = Backend()
    now = datetime.datetime.now()
    org = models.Organization.objects.create(name="Test org", created=now, updated=now)
    assert backend.get_object(models.Organization, org.id) == org


@pytest.mark.django_db
def test_get_objects(db):
    backend = Backend()
    now = datetime.datetime.now()
    orgs = []
    for i in range(0, 4):
        orgs.append(
            models.Organization.objects.create(
                name="Test org {}".format(i), created=now, updated=now
            )
        )
    assert [o for o in backend.get_objects(models.Organization, [1, 2])] == orgs[:2]
    assert [o for o in backend.get_objects(models.Organization)] == orgs


@pytest.mark.django_db
def test_get_object_by(db):
    backend = Backend()
    now = datetime.datetime.now()
    org = models.Organization.objects.create(name="Test org", created=now, updated=now)
    assert backend.get_object_by(models.Organization, "name", "Test org") == org


@pytest.mark.django_db
def test_get_objects_by(db):
    backend = Backend()
    now = datetime.datetime.now()
    orgs = []
    for i in range(0, 4):
        orgs.append(
            models.Organization.objects.create(
                name="Test org {}".format(i), created=now, updated=now
            )
        )
    assert [
        o for o in backend.get_objects_by(models.Organization, "name", "Test org 0")
    ] == orgs[:1]
    assert [
        o for o in backend.get_objects_by(models.Organization, "name", "Test org x")
    ] == []


@pytest.mark.django_db
def test_create_object(db):
    backend = Backend()
    now = datetime.datetime.now()
    org = backend.create_object(
        models.Organization, name="Test org", created=now, updated=now
    )
    assert org.id == 1
    assert org.name == "Test org"


@pytest.mark.django_db
def test_get_fields(db):
    backend = Backend()
    fields = backend.get_fields(models.Organization)
    assert fields == models.Organization._meta.get_fields()


@pytest.mark.django_db
def test_get_field(db):
    backend = Backend()
    field = backend.get_field(models.Organization, "name")
    assert field == models.Organization._meta.get_field("name")


@pytest.mark.django_db
def test_get_field_concrete(db):
    backend = Backend()
    related_model = backend.get_field_concrete(models.Network, "org")
    assert related_model == models.Organization


@pytest.mark.django_db
def test_is_field_related(db):
    backend = Backend()
    related_model = backend.get_field_concrete(models.Network, "org")
    assert related_model == models.Organization


@pytest.mark.django_db
def test_set_relation_many_to_many(db):
    backend = Backend()
    now = datetime.datetime.now()
    org = backend.create_object(
        models.Organization, name="Test org", created=now, updated=now
    )
    org2 = backend.create_object(
        models.Organization, name="Test org 2", created=now, updated=now
    )
    nets = [
        models.Network.objects.create(
            asn=i, name="Net {}".format(i), created=now, updated=now, org=org
        )
        for i in range(1, 4)
    ]
    backend.set_relation_many_to_many(org2, "net_set", nets)

    assert [n for n in org2.net_set.all()] == nets


@pytest.mark.django_db
def test_clean(db):
    backend = Backend()
    net = models.Network()
    with pytest.raises(Backend.validation_error()):
        backend.clean(net)


@pytest.mark.django_db
def test_save(db):
    backend = Backend()
    now = datetime.datetime.now()
    org = models.Organization(name="Test org", created=now, updated=now)
    backend.save(org)
    assert org.id == 1


def test_convert_field():
    backend = Backend()
    assert isinstance(backend.convert_field(models.Facility, "latitude", 1.0), Decimal)


@pytest.mark.django_db
def test_detect_missing_relations(db):
    backend = Backend()
    now = datetime.datetime.now()
    net = models.Network(name="Test net", org_id=1, created=now, updated=now)
    with pytest.raises(Backend.validation_error()) as exc:
        backend.clean(net)

    info = backend.detect_missing_relations(net, exc.value)
    expected = defaultdict(set)
    expected[backend.get_resource(models.Organization)] = {1}
    assert dict(info) == dict(expected)


@pytest.mark.django_db
def test_detect_uniqueness_error(db):
    backend = Backend()
    now = datetime.datetime.now()
    org = models.Organization.objects.create(name="Test org", created=now, updated=now)
    org2 = models.Organization(name="Test org", created=now, updated=now)
    with pytest.raises(IntegrityError) as exc:
        backend.save(org2)

    info = backend.detect_uniqueness_error(exc.value)
    assert info == ["name"]



from setuptools import find_packages, setup


def read_file(name):
    with open(name) as fobj:
        return fobj.read().strip()


LONG_DESCRIPTION = read_file("README.md")
VERSION = read_file("Ctl/VERSION")
REQUIREMENTS = read_file("Ctl/requirements.txt").split("\n")
TEST_REQUIREMENTS = read_file("Ctl/requirements-test.txt").split("\n")


setup(
    name="django-peeringdb",
    version=VERSION,
    author="PeeringDB",
    author_email="support@peeringdb.com",
    description="PeeringDB models and local synchronization for Django",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    license="LICENSE.txt",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Internet",
    ],
    packages=find_packages(),
    include_package_data=True,
    url="https://github.com/peeringdb/django-peeringdb",
    download_url="https://github.com/peeringdb/django-peeringdb/archive/{}.zip".format(
        VERSION
    ),
    install_requires=REQUIREMENTS,
    tests_require=TEST_REQUIREMENTS,
    zip_safe=True,
)



from django.core.exceptions import ValidationError
from django.test import TestCase
import ipaddress
import pytest

from django_peeringdb.models import (
    URLField,
    LG_URLField,
)
from tests.models import FieldModel, LG_FieldModel


class FieldTests(TestCase):
    """ test model functionality """

    def test_init(self):
        new = URLField()

    def test_url(self):
        model = FieldModel()

        model.url = "http://example.com"
        model.full_clean()

        model.url = "https://example.com"
        model.full_clean()

        with pytest.raises(ValidationError):
            model.url = "invalid"
            model.full_clean()


class LG_FieldTests(TestCase):
    """ test model functionality """

    def test_init(self):
        new = LG_URLField()

    def test_url(self):
        model = LG_FieldModel()

        model.url = "telnet://example.com"
        model.full_clean()

        model.url = "ssh://user@example.com"
        model.full_clean()

        with pytest.raises(ValidationError):
            model.url = "invalid"
            model.full_clean()



from django.db import models
from django_peeringdb.models import URLField, LG_URLField


class FieldModel(models.Model):
    url = URLField(null=True, blank=True)

    class Meta:
        app_label = "django_peeringdb.tests"


class LG_FieldModel(models.Model):
    url = LG_URLField(null=True, blank=True)

    class Meta:
        app_label = "django_peeringdb.tests"
